/** include files **/
#include "tscsincrorailnet.h"  // base header
#include "message.h"           // class InternalMessage 
#include "mainsimu.h"          // class Simulator
#include "distri.h"            // class Distribution 
#include "strutil.h"           // str2Value( ... )

/*******************************************************************
* Function Name: TSCSincroRailnet
* Description: constructor
********************************************************************/
TSCSincroRailnet::TSCSincroRailnet( const string &name )
: Atomic( name )

{
	try
	{
		dist = Distribution::create( MainSimulator::Instance().getParameter( description(), "distribution" ) );
		MASSERT( dist ) ;
		for ( register int i = 0; i < dist->varCount(); i++ )
		{
			string parameter( MainSimulator::Instance().getParameter( description(), dist->getVar( i ) ) ) ;
			dist->setVar( i, str2Value( parameter ) ) ;
		}

		if( MainSimulator::Instance().existsParameter( description(), "railnets" ) )
			railnets = str2Int( MainSimulator::Instance().getParameter( description(), "railnets" ) );
            else 
			railnets = 0;

		if (railnets == 0) {
                MTHROW(MException("TSCSincroRailnet: model " + description() + " has railnets = 0 !" ));
                }

	} catch( InvalidDistribution &e )
	{
		e.addText( "The model " + description() + " has distribution problems!" ) ;
		e.print(cerr);
		MTHROW( e ) ;
	} catch( MException &e )
	{
		MTHROW( e ) ;
	}

	int i;
	string s;
	char buff[5];
	for (i=0; i <railnets; i++){
		s = "y_vt_tren";
		s += itoa(i,buff,10);
		y_vt_tren.insert(map<int,Port*>::value_type(i,&addOutputPort(s)));
	}

}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &TSCSincroRailnet::initFunction()
{
      railnet = 0;
	holdIn( active, Time::Zero ) ;
	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &TSCSincroRailnet::internalFunction( const InternalMessage & )
{
	holdIn( active, Time( static_cast< float >( fabs( distribution().get() ) ) ) ) ;
      railnet = railnet + 1;
      if (railnet == railnets)
         railnet = 0;
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &TSCSincroRailnet::outputFunction( const InternalMessage &msg )
{
  int i;
  for (i=0; i <railnets; i++){
      if (i==railnet)
   		sendOutput( msg.time(), *(y_vt_tren[i]), 0 );   // 0 = train
	else
   		sendOutput( msg.time(), *(y_vt_tren[i]), 1 );   // 1 = no train
  }
  return *this ;
}


/*******************************************************************
* Function Name: externalFunction
********************************************************************/
Model &TSCSincroRailnet::externalFunction( const ExternalMessage &msg ){
	throw InvalidMessageException();
}


TSCSincroRailnet::~TSCSincroRailnet()
{
	delete dist;
      delete &y_vt_tren;
}