#include "Crossing.h"

void Crossing::setIdent (string id)
{
  this->ident = id;
}

void Crossing::setPoint (Point p)
{
  this->point = p;
}

void Crossing::setTLight (string tl)
{
  this->tLight = tl;
}

void Crossing::setHole (string h)
{
  this->hole = h;
}

void Crossing::setSpeed (int sp)
{
  this->speed = sp;
}

void Crossing::setDelay (int d)
{
  this->delay = d;
}

void Crossing::setPout (int p)
{
  this->pout = p;
}

string Crossing::getIdent ()
{
  return this->ident;
}

Point Crossing::getPoint ()
{
  return this->point;
}

string Crossing::getTLight ()
{
  return this->tLight;
}

string Crossing::getHole ()
{
  return this->hole;
}

int Crossing::getSpeed ()
{
  return this->speed;
}

int Crossing::getQcell ()
{
  return this->qcell;
}

int Crossing::getDelay ()
{
  return this->delay;
}

int Crossing::getPout ()
{
  return this->pout;
}

void Crossing::setQcell ()
{
  list < Segment * >::iterator iterS;
  int q = 0;
  for (iterS = this->lSeg.begin (); iterS != this->lSeg.end (); iterS++)
    q += (*iterS)->getQlane ();
  this->qcell = q;
}

int Crossing::getQcellIn ()
{
  list < Segment * >::iterator iterS;
  int q = 0;
  for (iterS = this->lSeg.begin (); iterS != this->lSeg.end (); iterS++)
    if (this->isInSeg (*iterS))
      q += (*iterS)->getQlane ();
  return q;
}

int Crossing::getQcellOut ()
{
  list < Segment * >::iterator iterS;
  int q = 0;
  for (iterS = this->lSeg.begin (); iterS != this->lSeg.end (); iterS++)
    if (this->isOutSeg (*iterS))
      q += (*iterS)->getQlane ();
  return q;
}

int Crossing::getQSegIn ()
{
  list < Segment * >::iterator iterS;
  int q = 0;
  for (iterS = this->lSeg.begin (); iterS != this->lSeg.end (); iterS++)
    if (this->isInSeg (*iterS))
      q += 1;
  return q;
}

int Crossing::getQSegOut ()
{
  list < Segment * >::iterator iterS;
  int q = 0;
  for (iterS = this->lSeg.begin (); iterS != this->lSeg.end (); iterS++)
    if (this->isOutSeg (*iterS))
      q += 1;
  return q;
}


bool Crossing::isInSeg (Segment * s)
{
  return (this->point == s->getEndPoint ());
}

bool Crossing::isOutSeg (Segment * s)
{
  return (this->point == s->getStartPoint ());
}



void Crossing::addSeg (Segment * s)
{
  bool ubic;
  double inclS;
  double inclIterS;
  list < Segment * >::iterator iterS;

  if (s->getFirstPoint () == this->point)
    inclS = s->getFirstIncl ();
  else
    inclS = s->getSecondIncl ();

  ubic = false;
  iterS = this->lSeg.begin ();
  while (iterS != this->lSeg.end () && !ubic)
    {
      if ((*iterS)->getFirstPoint () == this->point)
	inclIterS = (*iterS)->getFirstIncl ();
      else
	inclIterS = (*iterS)->getSecondIncl ();
      if (inclIterS < inclS)
	iterS++;
      else
	ubic = true;
    }
  this->lSeg.insert (iterS, s);
}



list < Segment * >*Crossing::getLSeg ()
{
  return &(this->lSeg);
}

void Crossing::print ()
{
  cout << "Crossing " << this->ident << "|";
  this->point.print ();
  cout << "|";
  cout << this->speed << "|"
    << this->tLight << "|"
    << this->hole << "|" << this->qcell << "|" << this->delay << "|" << this->pout << "|";

  list < Segment * >::iterator iter;
  if (this->lSeg.size () != 0)
    for (iter = this->lSeg.begin (); iter != this->lSeg.end (); iter++)
      {
	cout << "\n   Segment(" << (*iter)->getIdent () << ")";
      };
}

bool Crossing::operator == (const Crossing & c) const 
{
  return (this->point == c.point);
}

bool Crossing::operator != (const Crossing & c) const
{
  return (this->point != c.point);
}

bool Crossing::operator < (const Crossing & c) const
{
  return (this->point < c.point);
}

bool Crossing::operator <= (const Crossing & c) const
{
  return (this->point <= c.point);
}

bool Crossing::operator > (const Crossing & c) const
{
  return (this->point > c.point);
}

bool Crossing::operator >= (const Crossing & c) const
{
  return (this->point >= c.point);
}


Segment * Crossing::getSegment (int n)
{
  list < Segment * >::iterator iterS;
  int q = 0;
  bool ubic = false;
  iterS = this->lSeg.begin ();
  while (iterS != this->lSeg.end () && !ubic)
    {
      if (n < q + (*iterS)->getQlane ())
        ubic = true;
      else
        {
          q += (*iterS)->getQlane ();
          iterS++;
        }
    }
  return *iterS;
}


int Crossing::getSegmentLane (int n)
{
  list < Segment * >::iterator iterS;
  int q = 0;
  bool ubic = false;
  iterS = this->lSeg.begin ();
  while (iterS != this->lSeg.end () && !ubic)
    {
      if (n < q + (*iterS)->getQlane ())
        ubic = true;
      else
        {
          q += (*iterS)->getQlane ();
          iterS++;
        }
    }
  return (n - q);
}
