#include "lineconvertor.h"

string LineConvertor::convertSegment (string line, Segment * s)
{
  char buff[5];

// Replace of simple variables
  line = convertLine (line, "&IDENTIF&", s->getIdent ());
  if (s->getStartCross () != NULL)
     line = convertLine (line, "&STARTCROSS_IDENTIF&", s->getStartCross ()->getIdent ());
  if (s->getEndCross () != NULL)
     line = convertLine (line, "&ENDCROSS_IDENTIF&", s->getEndCross ()->getIdent ());
  line = convertLine (line, "&WIDTH&", itoa (s->getQcell (), buff, 10));
  line = convertLine (line, "&HEIGHT&", itoa (s->getQlane (), buff, 10));
  line = convertLine (line, "&SPEED&", itoa (s->getSpeed (), buff, 10));
  line = convertLine (line, "&FIRST_LANE&", "0");
  line = convertLine (line, "&LAST_LANE&", itoa (s->getQlane () - 1, buff, 10));
  line = convertLine (line, "&FIRST_CELL&", "0");
  line = convertLine (line, "&LAST_CELL&", itoa (s->getQcell () - 1, buff, 10));
  line = convertLine (line, "&LAST_CELL-1&", itoa (s->getQcell () - 2, buff, 10));
  line = convertLine (line, "&DELAY&", itoa (s->getDelay (), buff, 10));

// Replace of multiple variables
  line = convertForLine (line, "&LANE&", s->getQlane ());
  line = convertForLine (line, "&CELL&", s->getQcell ());

  if (getVariable (line) != "")
    {
      TemplateName *tn;
      string m = "Error in model ";
      m += (tn->getTNSegment (s));
      m += " for segment ";
      m += s->getIdent ();
      m += " - Variable: " + getVariable (line) + " unknown";
      throw TException ("convertSegment", m);
    }

  return line;
}


string LineConvertor::convertJobsite (string line, Jobsite * j)
{
  char buff[5];
  line = convertLine (line, "&JOBSITE_CELLS&", j->getCells ());
  line = convertLine (line, "&DELAY&", itoa (j->getDelay (), buff, 10));
  return line;
}

string LineConvertor::convertHole (string line, Hole * h)
{
  char buff[5];
  line = convertLine (line, "&HOLE_CELL&", itoa (h->getDistance (), buff, 10));
  line = convertLine (line, "&HOLE_LANE&", itoa (h->getLane () - 1, buff, 10));
  line = convertLine (line, "&DELAY&", itoa (h->getDelay (), buff, 10));
  return line;
}

string LineConvertor::convertCtrElem (string line, CtrElem * c)
{
  char buff[5];
  line = convertLine (line, "&CONTROL_CELL&", itoa (c->getDistance (), buff, 10));
  line = convertLine (line, "&DELAY&", itoa (c->getDelay (), buff, 10));
  return line;
}


string LineConvertor::convertRail (string line, Segment * s, Railnet * r)
{
  char buff[5];
  line =
    convertLine (line, "&BEFORE_TRAIN_CELL&", itoa (r->findDistance (s->getIdent ()), buff, 10));
  line =
    convertLine (line, "&AFTER_TRAIN_CELL&", itoa (r->findDistance (s->getIdent ()) + 1, buff, 10));
  line = convertLine (line, "&RAIL_ORDER&", itoa (r->findRailOrder (s->getIdent ()) - 1, buff, 10));
  line = convertLine (line, "&RAIL_IDENTIF&", r->getIdent ());
  line = convertLine (line, "&RAIL_DELAY&", itoa (r->getDelay (), buff, 10));

  return line;
}


string LineConvertor::convertRailnet (string line, Railnet * r)
{
  string repLine, backupLine;
  char buff[5];

// Replace of simple variables
  line = convertLine (line, "&IDENTIF&", r->getIdent ());
  line = convertLine (line, "&(SEGMENTS)&", itoa (r->getQsegs (), buff, 10));

// Replace of multiple variables
  line = convertForLine (line, "&#SEGMENTS&", r->getQsegs ());

  if (line.find ("&SEGMENTS&") != string::npos)
    {
      repLine = "";
      list < pair < Segment *, int > >*lRail = r->getLRail ();
      list < pair < Segment *, int > >::iterator iter;

      backupLine = line;
      line = "";
      for (iter = lRail->begin (); iter != lRail->end (); iter++)
	{
	  repLine = backupLine;
	  repLine = convertLine (repLine, "&SEGMENTS&", (*iter).first->getIdent ());
	  line += repLine;
	}
    }

  if (getVariable (line) != "")
    {
      TemplateName *tn;
      string m = "Error in model ";
      m += (tn->getTNSincroRailnet ());
      m += " for railnet ";
      m += r->getIdent ();
      m += " - Variable: " + getVariable (line) + " unknown";
      throw TException ("convertRailnet", m);
    }
  return line;
}

string LineConvertor::convertCrossing (string line, Crossing * c)
{
  char buff[5];
// Replace of simple variables

  line = convertLine (line, "&IDENTIF&", c->getIdent ());
  line = convertLine (line, "&WIDTH&", itoa (c->getQcell (), buff, 10));
  line = convertLine (line, "&SPEED&", itoa (c->getSpeed (), buff, 10));
  line = convertLine (line, "&LAST_CELL&", itoa (c->getQcell () - 1, buff, 10));
  line = convertLine (line, "&LAST_CELL-1&", itoa (c->getQcell () - 1, buff, 10));
  line = convertLine (line, "&(IN)&", itoa (c->getQcellIn (), buff, 10));
  line = convertLine (line, "&(OUT)&", itoa (c->getQcellOut (), buff, 10));
  line = convertLine (line, "&(IN_SEGMENTS)&", itoa (c->getQSegIn (), buff, 10));
  line = convertLine (line, "&(OUT_SEGMENTS)&", itoa (c->getQSegOut (), buff, 10));
  line = convertLine (line, "&DELAY&", itoa (c->getDelay (), buff, 10));
  line = convertLine (line, "&POUT&", itoa (c->getPout (), buff, 10));


// Replace of multiple variables
  string repLine, backupLine;
  int i, sumSegLanes, qlane;
  list < Segment * >*lSeg = c->getLSeg ();
  list < Segment * >::iterator iter;

  if (line.find ("&IN&") != string::npos)
    {
      sumSegLanes = 0;
      backupLine = line;
      line = "";
      for (iter = lSeg->begin (); iter != lSeg->end (); iter++)
	{
	  qlane = (*iter)->getQlane ();
	  if (c->isInSeg (*iter))
	    {
	      for (i = sumSegLanes; i < sumSegLanes + qlane; i++)
		{
		  repLine = backupLine;
		  repLine = convertLine (repLine, "&IN&", itoa (i, buff, 10));
		  repLine = convertLine (repLine, "&IN_SEGMENT&", (*iter)->getIdent ());
		  repLine =
		    convertLine (repLine, "&SEG_LANE_IN&", itoa (c->getSegmentLane (i), buff, 10));
		  repLine =
		    convertLine (repLine, "&SEG_CELL_IN&",
				 itoa ((*iter)->getQcell () - 1, buff, 10));
		  line += repLine;
		}
	    }
	  sumSegLanes += qlane;
	}
    }

  if (line.find ("&#IN&") != string::npos)
    {
      sumSegLanes = 0;
      backupLine = line;
      line = "";
      for (iter = lSeg->begin (); iter != lSeg->end (); iter++)
	{
	  qlane = (*iter)->getQlane ();
	  if (c->isInSeg (*iter))
	    {
	      for (i = sumSegLanes; i < sumSegLanes + qlane; i++)
		{
		  repLine = backupLine;
		  repLine = convertLine (repLine, "&#IN&", itoa (i, buff, 10));
		  repLine = convertLine (repLine, "&IN_SEGMENT&", (*iter)->getIdent ());
		  repLine =
		    convertLine (repLine, "&SEG_LANE_IN&", itoa (c->getSegmentLane (i), buff, 10));
		  repLine =
		    convertLine (repLine, "&SEG_CELL_IN&",
				 itoa ((*iter)->getQcell () - 1, buff, 10));
		  line += repLine;
		}
	      sumSegLanes += qlane;
	    }
	}
    }

  if (line.find ("&#IN_SEGMENTS&") != string::npos)
    {
      i = 0;
      backupLine = line;
      line = "";
      for (iter = lSeg->begin (); iter != lSeg->end (); iter++)
	{
	  if (c->isInSeg (*iter))
	    {
	      repLine = backupLine;
	      repLine = convertLine (repLine, "&#IN_SEGMENTS&", itoa (i, buff, 10));
	      repLine = convertLine (repLine, "&IN_SEGMENT&", (*iter)->getIdent ());
	      repLine =
		convertLine (repLine, "&SEG_LANE_IN&", itoa (c->getSegmentLane (i), buff, 10));
	      repLine =
		convertLine (repLine, "&SEG_CELL_IN&", itoa ((*iter)->getQcell () - 1, buff, 10));
	      line += repLine;
	      i += 1;
	    }
	}
    }

  if (line.find ("&IN_SEGMENTS&") != string::npos)
    {
      sumSegLanes = 0;
      backupLine = line;
      line = "";
      for (iter = lSeg->begin (); iter != lSeg->end (); iter++)
	{
	  qlane = (*iter)->getQlane ();
	  if (c->isInSeg (*iter))
	    {
	      for (i = sumSegLanes; i < sumSegLanes + qlane; i++)
		{
		  repLine = backupLine;
		  repLine = convertLine (repLine, "&IN_SEGMENTS&", (*iter)->getIdent ());
		  repLine =
		    convertLine (repLine, "&SEG_LANE_IN&", itoa (c->getSegmentLane (i), buff, 10));
		  repLine =
		    convertLine (repLine, "&SEG_CELL_IN&",
				 itoa ((*iter)->getQcell () - 1, buff, 10));
		  line += repLine;
		}
	    }
	  sumSegLanes += qlane;
	}
    }

  if (line.find ("&OUT&") != string::npos)
    {
      sumSegLanes = 0;
      backupLine = line;
      line = "";
      for (iter = lSeg->begin (); iter != lSeg->end (); iter++)
	{
	  qlane = (*iter)->getQlane ();
	  if (c->isOutSeg (*iter))
	    {
	      for (i = sumSegLanes; i < sumSegLanes + qlane; i++)
		{
		  repLine = backupLine;
		  repLine = convertLine (repLine, "&OUT&", itoa (i, buff, 10));
		  repLine = convertLine (repLine, "&OUT_SEGMENT&", (*iter)->getIdent ());
		  repLine =
		    convertLine (repLine, "&SEG_LANE_OUT&", itoa (c->getSegmentLane (i), buff, 10));
		  repLine = convertLine (repLine, "&SEG_CELL_OUT&", "0");
		  line += repLine;
		}
	    }
	  sumSegLanes += qlane;
	}
    }

  if (line.find ("&#OUT&") != string::npos)
    {
      sumSegLanes = 0;
      backupLine = line;
      line = "";
      for (iter = lSeg->begin (); iter != lSeg->end (); iter++)
	{
	  qlane = (*iter)->getQlane ();
	  if (c->isOutSeg (*iter))
	    {
	      for (i = sumSegLanes; i < sumSegLanes + qlane; i++)
		{
		  repLine = backupLine;
		  repLine = convertLine (repLine, "&#OUT&", itoa (i, buff, 10));
		  repLine = convertLine (repLine, "&OUT_SEGMENT&", (*iter)->getIdent ());
		  repLine =
		    convertLine (repLine, "&SEG_LANE_OUT&", itoa (c->getSegmentLane (i), buff, 10));
		  repLine = convertLine (repLine, "&SEG_CELL_OUT&", "0");
		  line += repLine;
		}
	      sumSegLanes += qlane;
	    }
	}
    }

  if (line.find ("&#OUT_SEGMENTS&") != string::npos)
    {
      i = 0;
      backupLine = line;
      line = "";
      for (iter = lSeg->begin (); iter != lSeg->end (); iter++)
	{
	  qlane = (*iter)->getQlane ();
	  if (c->isOutSeg (*iter))
	    {
	      repLine = backupLine;
	      repLine = convertLine (repLine, "&#OUT_SEGMENTS&", itoa (i, buff, 10));
	      repLine = convertLine (repLine, "&OUT_SEGMENT&", (*iter)->getIdent ());
	      repLine =
		convertLine (repLine, "&SEG_LANE_OUT&", itoa (c->getSegmentLane (i), buff, 10));
	      repLine = convertLine (repLine, "&SEG_CELL_OUT&", "0");
	      line += repLine;
	      i += 1;
	    }
	}
    }

  if (line.find ("&OUT_SEGMENTS&") != string::npos)
    {
      sumSegLanes = 0;
      backupLine = line;
      line = "";
      for (iter = lSeg->begin (); iter != lSeg->end (); iter++)
	{
	  if (c->isOutSeg (*iter))
	    {
	      for (i = sumSegLanes; i < sumSegLanes + qlane; i++)
		{
		  repLine = backupLine;
		  repLine = convertLine (repLine, "&OUT_SEGMENTS&", (*iter)->getIdent ());
		  repLine =
		    convertLine (repLine, "&SEG_LANE_OUT&", itoa (c->getSegmentLane (i), buff, 10));
		  repLine = convertLine (repLine, "&SEG_CELL_OUT&", "0");
		  repLine =
		    convertLine (repLine, "&SEG_CELL_OUT&",
				 itoa ((*iter)->getQcell () - 1, buff, 10));
		  line += repLine;
		}
	    }
	  sumSegLanes += qlane;
	}
    }

  if (line.find ("&CELL&") != string::npos)
    {
      sumSegLanes = 0;
      backupLine = line;
      line = "";
      for (iter = lSeg->begin (); iter != lSeg->end (); iter++)
	{
	  qlane = (*iter)->getQlane ();
	  for (i = sumSegLanes; i < sumSegLanes + qlane; i++)
	    {
	      repLine = backupLine;
	      repLine = convertLine (repLine, "&CELL&", itoa (i, buff, 10));
	      repLine = convertLine (repLine, "&SEG_LANE&", itoa (c->getSegmentLane (i), buff, 10));
	      if (c->isInSeg (*iter))
		repLine =
		  convertLine (repLine, "&SEG_CELL&", itoa ((*iter)->getQcell () - 1, buff, 10));
	      else
		repLine = convertLine (repLine, "&SEG_CELL&", "0");
	      line += repLine;
	    }
	  sumSegLanes += qlane;
	}
    }

  if (getVariable (line) != "")
    {
      TemplateName *tn;
      string m = "Error in model ";
      m += (tn->getTNCrossing ());
      m += " for crossing ";
      m += c->getIdent ();
      m += " - Variable: " + getVariable (line) + " unknown";
      throw TException ("convertCrossing", m);
    }

  return line;
}


string LineConvertor::getVariable (string l)
{
  string laux;
  int iaux;

  laux = l;
  iaux = laux.find ("&");
  if (iaux != string::npos)
    {
      laux.erase (0, iaux + 1);
      laux.erase (laux.find ("&"), laux.size () - laux.find ("&"));
      return ("&" + laux + "&");
    }
  return ("");
}


string LineConvertor::convertLine (string line, string transtring, string repstring)
{
  int ubic = line.find (transtring);
  while (ubic != string::npos)
    {
      line.replace (ubic, transtring.size (), repstring);
      ubic = line.find (transtring);
    }
  return line;
}


string LineConvertor::convertForLine (string line, string transtring, int limit)
{
  int i;
  string repLine = "";
  char buff[5];
  if (line.find (transtring) != string::npos)
    for (i = 0; i < limit; i++)
      {
	repLine += line;
	repLine = convertLine (repLine, transtring, itoa (i, buff, 10));
      }
  else
    repLine = line;
  return repLine;

}
