%{
#include "planyacc.h"
extern int errorline;   
%}

%option yylineno
%option noyywrap       
DIGIT    [0-9]
SYMBOL   [-_]
LETTER   [A-Za-z]
ID       {LETTER}({LETTER}|{DIGIT}|{SYMBOL})*

%%

{DIGIT}+             {planlval.s = yytext;
                      return TOK_NUMBER;}


"("|")"|","|"="|":"  {planlval.s = yytext;return *yytext;}


"begin"              {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_BEGIN;
                     }

"end"                {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_END;
                     }

"segments"           {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_SEGMENTS;
                     }

"crossings"          {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_CROSSINGS;
                     }

"go"                 {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_GO;
                     }

"back"               {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_BACK;
                     }

"straight"           {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_STRAIGHT;
                     }

"curve"              {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_CURVE;
                     }

"withTL"             {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_WITH_TLIGHT;
                     }

"withoutTL"          {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_WITHOUT_TLIGHT;
                     }

"withHole"           {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_WITH_HOLE;
                     }

"withoutHole"        {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_WITHOUT_HOLE;
                     }

"railnets"           {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_RAILNETS;
                     }

"in"                 {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_IN;
                     }

"jobsites"           {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_JOBSITES;
                     }

"holes"              {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_HOLES;
                     }

"ctrElements"        {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_CTRELEMS;
                     }

"sawhorse"           {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_CTRTYPE1;
                     }

"depression"         {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_CTRTYPE2;
                     }

"intersection"       {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_CTRTYPE3;
                     }

"saw"                {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_CTRTYPE4;
                     }

"stop"               {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_CTRTYPE5;
                     }

"school"             {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_CTRTYPE6;
                     }

"parkNone"           {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_PARKNONE;
                     }

"parkLeft"           {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_PARKLEFT;
                     }

"parkRight"          {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_PARKRIGHT;
                     }

"parkBoth"           {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_PARKBOTH;
                     }

{ID}                 {planlval.s = yytext;
                      errorline = yylineno;
                      return TOK_IDENTIF;
                     }

"//"(.)*\n  /* comment of single line */


"/*"        {/* comment of multiples lines */
            register int c;
            for ( ; ; ){
                /* eat up text of comment */
                while ( (c = yyinput()) != '*' &&  c != EOF );
                if ( c == '*' ){
                    while ( (c = yyinput()) == '*' );
                    if ( c == '/' ) break;    /* found the end */
                    }
                if ( c == EOF ){
                    printf("EOF in comment\n" );
                    break;
                    }
                }
            }


[ \t\n]+          /* eliminates space,tab,nl */

.           { /* character not expected */
             planlval.s = yytext; 
             errorline = yylineno;
             return  TOK_ERROR;
            }

%%
