/** include files **/
#include "tscconsumer.h"       // base header
#include "message.h"           // class InternalMessage 
#include "parsimu.h"          // class Simulator
#include "strutil.h"           // str2Value( ... )
#include "stdlib.h"
#include "atomicstate.h"

/*******************************************************************
* Function Name: TSCConsumer
* Description: constructor
********************************************************************/
TSCConsumer::TSCConsumer( const string &name )
: Atomic( name )
, quantity( addOutputPort( "quantity" ) )
, quantityAcum( addOutputPort( "quantityAcum" ) )
{
	frecuence = "0:1:0:0" ;
	timeUnit = "0:0:1:0" ;
	if( ParallelMainSimulator::Instance().existsParameter( description(), "frecuence" ) )
		frecuence = ParallelMainSimulator::Instance().getParameter( description(), "frecuence" ) ;

	if( ParallelMainSimulator::Instance().existsParameter( description(), "timeUnit" ) )
		timeUnit = ParallelMainSimulator::Instance().getParameter( description(), "timeUnit" ) ;

	if( ParallelMainSimulator::Instance().existsParameter( description(), "lanes" ) )
		lanes = str2Int( ParallelMainSimulator::Instance().getParameter( description(), "lanes" ) );
	else	
		lanes = 0;

	if (lanes == 0) {
          MTHROW(MException("TSCConsumer: model " + description() + " has lanes = 0 !" ));
         }
	
	int i;
	string s;
	char buff[5];
	for (i=0; i <lanes; i++){
		//s = "x_t_hayauto";
		//s += itoa(i,buff,10);
		sprintf(buff, "x_t_hayauto%d", i);
                                  s = buff;
		x_t_hayauto.insert(map<int,Port*>::value_type(i,&addInputPort(s)));
	}
}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &TSCConsumer::initFunction()
{
	qty = 0;
	holdIn( AtomicState::active, frecuence );
	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &TSCConsumer::internalFunction( const InternalMessage & )
{
	holdIn( AtomicState::active, frecuence );
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &TSCConsumer::outputFunction( const InternalMessage &msg )
{
	float time( msg.time().asMsecs() / timeUnit.asMsecs() ) ;

	sendOutput( msg.time(), quantity, qty / time ) ;
	sendOutput( msg.time(), quantityAcum, qty ) ;

	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
********************************************************************/
Model &TSCConsumer::externalFunction( const ExternalMessage &msg ){
	qty = qty + 1 ;
	return *this ;
}

TSCConsumer::~TSCConsumer()
{
      delete &x_t_hayauto;
}
