/** include files **/
#include "tscrailnet.h"      // base header
#include "message.h"           // class InternalMessage 
#include "parsimu.h"          // class Simulator
#include "strutil.h"           // str2Value( ... )
#include "stdlib.h"
#include "atomicstate.h"

/*******************************************************************
* Function Name: TSCRailnet
* Description: constructor
********************************************************************/
TSCRailnet::TSCRailnet( const string &name )
: Atomic( name )
, x_si_tren( addInputPort( "x_si_tren" ) )
{
	if( ParallelMainSimulator::Instance().existsParameter( description(), "lanes" ) )
		lanes = str2Int( ParallelMainSimulator::Instance().getParameter( description(), "lanes" ) );
	else 
		lanes = 0;
	if (lanes == 0) {
          MTHROW(MException("TSCRailnet: model " + description() + " has lanes = 0 !" ));
          }

	int i;
	string s1,s2;
	char buff[5];
	for (i=0; i <lanes; i++){
		//s1 = "y_t_tren";
		//s1 += itoa(i,buff,10);
		 sprintf(buff, "y_t_tren%d", i);
                                     s1= buff;
		s2 = s1 + "bt";
		y_t_tren_bt.insert(map<int,Port*>::value_type(i,&addOutputPort(s2)));
		s2 = s1 + "at";
		y_t_tren_at.insert(map<int,Port*>::value_type(i,&addOutputPort(s2)));
	}

}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &TSCRailnet::initFunction()
{
      train = 1;
	holdIn( AtomicState::active, VTime::Zero ) ;
	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &TSCRailnet::internalFunction( const InternalMessage & )
{
	this->passivate();
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &TSCRailnet::outputFunction( const InternalMessage &msg )
{
  int i;
  for (i=0; i <lanes; i++){
	sendOutput( msg.time(), *(y_t_tren_bt[i]), train ) ;
	sendOutput( msg.time(), *(y_t_tren_at[i]), train ) ;
  }
  return *this ;
}


/*******************************************************************
* Function Name: externalFunction
********************************************************************/
Model &TSCRailnet::externalFunction( const ExternalMessage &msg ){
	train = static_cast< int >( msg.value());
	holdIn( AtomicState::active, VTime::Zero ) ;
	return *this ;
}


TSCRailnet::~TSCRailnet()
{
      delete &y_t_tren_bt;
      delete &y_t_tren_at;
}
