/** include files **/
#include "tsctrafficlight.h"   // base header
#include "message.h"           // class InternalMessage 
#include "parsimu.h"          // class Simulator
#include "strutil.h"           // str2Value( ... )
#include "stdlib.h"
#include "atomicstate.h"

/*******************************************************************
* Function Name: TSCTrafficLight
* Description: constructor
********************************************************************/
TSCTrafficLight::TSCTrafficLight( const string &name )
: Atomic( name )
, x_si_luz( addInputPort( "x_si_luz" ) )
, x_t_luz( addInputPort( "x_t_luz" ) )
{
	if( ParallelMainSimulator::Instance().existsParameter( description(), "lanes" ) )
		lanes = str2Int( ParallelMainSimulator::Instance().getParameter( description(), "lanes" ) );
	else 
		lanes = 0;
	if (lanes == 0) {
         MTHROW(MException("TSCTrafficLight: model " + description() + " has lanes = 0 !" ));
         }

	int i;
	string s;
	char buff[5];
	for (i=0; i <lanes; i++){
		//s = "y_t_luz";
		//s += itoa(i,buff,10);
		   sprintf(buff, "y_t_luz%d", i);
                                      s = buff;
		y_t_luz.insert(map<int,Port*>::value_type(i,&addOutputPort(s)));
	}

}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &TSCTrafficLight::initFunction()
{
      color = 1;
	this->passivate();
	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &TSCTrafficLight::internalFunction( const InternalMessage & )
{
	this->passivate();
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &TSCTrafficLight::outputFunction( const InternalMessage &msg )
{
  int i;
  for (i=0; i <lanes; i++)
	sendOutput( msg.time(), *(y_t_luz[i]), color ) ;
  return *this ;
}


/*******************************************************************
* Function Name: externalFunction
********************************************************************/
Model &TSCTrafficLight::externalFunction( const ExternalMessage &msg ){
	if( msg.port() == x_si_luz )
	{
		color = static_cast<int>(msg.value());  
//		holdIn( AtomicState::active, TVime::Zero ) ;        // semaforo atlas 
   		this->passivate();                    // semaforo alternativo
	}
	else
		holdIn( AtomicState::active, VTime::Zero ) ;
	return *this ;
}


TSCTrafficLight::~TSCTrafficLight()
{
      delete &y_t_luz;
}
