#ifndef __CROSSING_H
#define __CROSSING_H

#include <istream>
#include <list>
#include <string>
#include "point.h"
#include "segment.h"

//forward declarations
class Segment;
class Point;


class Crossing {
public:
    void      setIdent(std::string);
    void      setPoint(Point);
    void      setTLight(std::string);
    void      setHole(std::string);
    void      setSpeed(int);
    void      setQcell();
    void      setDelay(int);
    void      setPout(int);
    std::string    getIdent();
    Point     getPoint();
    std::string    getTLight();
    std::string    getHole();
    int       getSpeed();
    int       getQcell();
    int       getQcellIn();
    int       getQcellOut();
    int       getQSegIn();
    int       getQSegOut();
    int       getDelay();
    int       getPout();
    void      print();                 //print the crossing info to the stdin
    bool      isInSeg(Segment*);       //returns true if the segment enters to the crossing
    bool      isOutSeg(Segment*);      //returns true if the segment exits from the crossing
    void      addSeg(Segment *);       //add segment to the crossing
    std::list<Segment*>* getLSeg();         //get all the segments in the crossing  

    Segment*  getSegment(int);         //da el segmento que se acopla en esa celda
    int       getSegmentLane(int);    //da el lane del segmento que se acopla en esa celda


    bool operator == ( const Crossing & ) const;
    bool operator != ( const Crossing & ) const;
    bool operator <  ( const Crossing & ) const;
    bool operator <= ( const Crossing & ) const;
    bool operator >  ( const Crossing & ) const;
    bool operator >= ( const Crossing & ) const;

protected:
    std::string   ident;
    Point    point;
    std::string   tLight;                  // indicates the presence of traffic light
    std::string   hole;                    // indicates the presence of hole
    int      speed;
    int      qcell;
    int      delay;
    int      pout;
    std::list<Segment*> lSeg;                 // list of segments in the crossing

private:
};




#endif /* __CROSSING_H */
